using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Pong
{
    class Ball
    {
        private Texture2D Texture;
        private Vector2 position = Vector2.Zero;
        private Vector2 speed = new Vector2(350.0f, 000.0f);
        private int TurboSpeedX = 700;
        private int TurboSpeedY = 900;
        private bool TurboOn = false;

        private int MaxSpeedX = 500;
        private int MaxSpeedY = 700;

        public Vector2 Position
        {
            get { return position; }
            set { position = value; }
        }
        public Vector2 Speed
        {
            get { return speed; }
            set { speed = value; }
        }

        //Constructor
        //***********
        public Ball(Texture2D Texture)
        {
            this.Texture = Texture;
        }

        //Change ball's texture
        //*********************
        public void NewText(Texture2D T)
        {
            this.Texture = T;
        }

        //Turbo Mode On
        //*************
        public void TurboBall(float TX, float TY)
        {
            TurboOn = true;
            if (speed.X > 1)
                speed.X = TurboSpeedX * TX;
            else if (speed.X < 1)
                speed.X = TurboSpeedX * TX * -1;
            if (speed.Y > 1)
                speed.Y = TurboSpeedY * TY;
            else if (speed.Y < 1)
                speed.Y = TurboSpeedY * TY * -1;
        }

        //Turbo Mode Off
        //**************
        public void TurboBallOff()
        {
            TurboOn = false;
            if (speed.X > 1)
                speed.X = 300;
            else if (speed.X < 1)
                speed.X = -300;
            if (speed.Y > 1)
                speed.Y = 500;
            else if (speed.Y < 1)
                speed.Y = -500;
        }

        //Bounces the ball
        //****************
        public void Bounce(float ModX, float ModY)
        {
            if (speed.X > 0)
                speed.X = (speed.X + ModX) * -1;
            else
                speed.X = (speed.X - ModX) * -1;
            if (speed.Y > 0)
                speed.Y = (speed.Y + ModY);
            else
                speed.Y = (speed.Y - ModY);
        }
        public void BounceAwayTop(float ModX, float ModY)
        {
            if (speed.X > 0)
                speed.X = (speed.X - ModX) * -1;
            else
                speed.X = (speed.X + ModX) * -1;
            if (speed.Y > 0)
                speed.Y = (speed.Y + ModY) * -1;
            else
                speed.Y = (speed.Y - ModY);
        }
        public void BounceAwayBottom(float ModX, float ModY)
        {
            if (speed.X > 0)
                speed.X = (speed.X - ModX) * -1;
            else
                speed.X = (speed.X + ModX) * -1;
            if (speed.Y < 0)
                speed.Y = (speed.Y - ModY) * -1;
            else
                speed.Y = (speed.Y + ModY);
        }

        //Updates the ball
        //****************
        public void Update(GameTime gameTime, GraphicsDeviceManager g, Rectangle windowBounds, SoundEffect bounceBlipSound)
        {
            //reset speeds...
            if (speed.X >= MaxSpeedX && !TurboOn && speed.X > 1)
                speed.X = 300;
            else if (speed.X <= (MaxSpeedX * -1) && !TurboOn && speed.X < 1)
                speed.X = -300;
            if (speed.Y >= MaxSpeedY && !TurboOn && speed.Y > 1)
                speed.Y = 500;
            else if (speed.Y <= (MaxSpeedY * -1) && !TurboOn && speed.Y < 1)
                speed.Y = -500;

            position += speed * (float)gameTime.ElapsedGameTime.TotalSeconds;

            int BMaxY = g.GraphicsDevice.Viewport.Height - Texture.Height;
            int BMinY = 50;

            if (position.Y > BMaxY)
            {
                bounceBlipSound.Play();
                speed.Y *= -1;
                position.Y = BMaxY;
            }
            else if (position.Y < BMinY)
            {
                bounceBlipSound.Play();
                speed.Y *= -1;
                position.Y = BMinY;
            }
        }

        //Draws the ball
        //**************
        public void Draw(SpriteBatch SP)
        {
            SP.Draw(Texture, position, Color.White);
        }

        //Returns a rectangle around the object
        //*************************************
        public Rectangle Rect
        {
            get
            {
                return new Rectangle(
                    (int)position.X, (int)position.Y,
                    Texture.Width, Texture.Height);
            }
        }

        //Restart position of the ball...
        //*******************************
        public void Restart(Rectangle windowBounds)
        {
            System.Random generator = new System.Random();
            int rand;

            position = new Vector2(windowBounds.Width / 2 - Texture.Width, windowBounds.Height / 2 - Texture.Height);

            rand = generator.Next();

            if (rand % 2 == 0)
                speed = new Vector2(250.0f, 000.0f);
            else
                speed = new Vector2(-250.0f, 000.0f);
        }
    }
}
