﻿#region Using Statements
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Pong
{
    class Menu
    {
        private static int MAX = 20;
        private int menuItemCount;
        private int curMenuItem;
        private string[] menuItems;
        private Vector2[] pos;
        private double[] scale;
        private Color unselected;
        private Color selected;
        private SpriteFont font;
        private bool boolShowInstructions;
        private bool boolShowCredits;
        private string[] instructions = new string[MAX];
        private string[] credits = new string[MAX];
        private Texture2D Texture;
        private Vector2 menuPos = Vector2.Zero;

        public Menu(Color unslectedColor, Color selectedColor, SpriteFont sp)
        {
            font = sp;
            menuItems = new string[MAX];
            pos = new Vector2[MAX];
            scale = new double[MAX];
            unselected = unslectedColor;
            selected = selectedColor;
            menuItemCount = 0;
            curMenuItem = 0;
            boolShowInstructions = false;
            boolShowCredits = false;
            menuPos = new Vector2(-115f, 0f);

            instructions[0] = "Press Escape at any time to return to the menu.";
            instructions[1] = "";
            instructions[2] = "Select the New Game option from the menu to begin, or to start a new game.";
            instructions[3] = "Select the Resume option from the menu during gameplay to return to the game.";
            instructions[4] = "Select the New Game option from the menu to begin.";
            instructions[5] = "Player 1 may move their paddle with W and S.";
            instructions[6] = "Player 2 may move their paddle with Up and Down.";
            instructions[7] = "";
            instructions[8] = "";
            instructions[9] = "";
            instructions[10] = "";
            instructions[11] = "";
            instructions[12] = "";
            instructions[13] = "";
            instructions[14] = "";
            instructions[15] = "";
            instructions[16] = "";
            instructions[17] = "";
            instructions[18] = "Press Escape to go back.";

            credits[0] = "http://bluwiki.com/go/XNA_Tutorials";
            credits[1] = "http://www.ziggyware.com/readarticle.php?article_id=188";
            credits[2] = "http://msdn.microsoft.com/en-us/library/bb195026(XNAGameStudio.10).aspx";
            credits[3] = "http://johnnygizmo.blogspot.com/2008/10/xna-part-6-audio-music.html";
            credits[4] = "";
            credits[5] = "";
            credits[6] = "";
            credits[7] = "";
            credits[8] = "";
            credits[9] = "";
            credits[10] = "";
            credits[11] = "";
            credits[12] = "";
            credits[13] = "";
            credits[14] = "";
            credits[15] = "";
            credits[16] = "";
            credits[17] = "";
            credits[18] = "Press Escape to go back.";
        }

        public void SetText(Texture2D T)
        {
            this.Texture = T;
        }

        public void addMenuItem(string name, Vector2 p)
        {
            if (menuItemCount < MAX)
            {
                menuItems[menuItemCount] = name;
                scale[menuItemCount] = 1.0f;
                pos[menuItemCount++] = p;
            }
        }

        public void changeMenuItem(string oldName, string newName)
        {
            for (int i = 0; i < menuItemCount; i++)
            {
                if (menuItems[i] == oldName)
                    menuItems[i] = newName;
            }
        }

        public void update(GameTime gameTime)
        {
            for (int x = 0; x < menuItemCount; x++)
            {
                while (menuItems[curMenuItem] == "")
                    curMenuItem++;

                if (x == curMenuItem)
                {
                    if (scale[x] < 2.0f)
                    {
                        scale[x] += 0.04 + 10.0f * gameTime.ElapsedGameTime.Seconds;
                    }
                }
                else if (scale[x] > 1.0f && x != curMenuItem)
                {
                    scale[x] -= 0.04 + 10.0f * gameTime.ElapsedGameTime.Seconds;
                }
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(Texture, menuPos, Color.White);

            if (boolShowInstructions == false && boolShowCredits == false)
            {
                for (int x = 0; x < menuItemCount; x++)
                {
                    if (x == curMenuItem)
                    {
                        Vector2 p = pos[x];
                        p.X -= (float)(22 * scale[x] / 2);
                        p.Y -= (float)(22 * scale[x] / 2);
                        spriteBatch.DrawString(font,
                                               menuItems[x],
                                               p,
                                               selected,
                                               0.0f,
                                               new Vector2(0, 0),
                                               (float)scale[x],
                                               SpriteEffects.None,
                                               0);
                    }
                    else
                    {
                        Vector2 p = pos[x];
                        p.X -= (float)(22 * scale[x] / 2);
                        p.Y -= (float)(22 * scale[x] / 2);
                        spriteBatch.DrawString(font,
                                               menuItems[x],
                                               p,
                                               unselected,
                                               0.0f,
                                               new Vector2(0, 0),
                                               (float)scale[x],
                                               SpriteEffects.None,
                                               0);
                    }
                }
            }
            else if (boolShowInstructions == true)
            {
                float instructionX = 20.0f;
                float instructionY = 20.0f;

                foreach (string instruction in instructions)
                {
                    if (instruction != null)
                        spriteBatch.DrawString(font, instruction, new Vector2(instructionX, instructionY), Color.Aquamarine);

                    instructionY += 30.0f;
                }
            }
            else if (boolShowCredits == true)
            {
                float creditX = 20.0f;
                float creditY = 20.0f;

                foreach (string credit in credits)
                {
                    if (credit != null)
                        spriteBatch.DrawString(font, credit, new Vector2(creditX, creditY), Color.Aquamarine);

                    creditY += 30.0f;
                }
            }
        }

        public int getSelectedNum()
        {
            return curMenuItem;
        }

        public string getSelectedName()
        {
            return menuItems[curMenuItem];
        }

        public void selectNext()
        {
            if (curMenuItem < menuItemCount - 1)
            {
                curMenuItem++;
            }
            else
            {
                curMenuItem = 0;
            }
            if (menuItems[curMenuItem] == "")
                selectNext();
        }

        public void selectPrev()
        {
            if (curMenuItem > 0)
            {
                curMenuItem--;
            }
            else
            {
                curMenuItem = menuItemCount - 1;
            }
            if (menuItems[curMenuItem] == "")
                selectPrev();
        }

        public void DisplayInstructions()
        {
            boolShowInstructions = true;
        }

        public void DisplayCredits()
        {
            boolShowCredits = true;
        }

        public void HideDisplays()
        {
            boolShowInstructions = false;
            boolShowCredits = false;
        }
    }
}
