using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Pong
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Menu gameMenu;
        bool boolPause = true;
        bool boolGameStarted = false;
        bool boolDisplaying = false;
        bool boolIntroSoundPlayed = false;
        string winner = "";

        int TurboCount = 0;
        bool Turbo = false;

        System.Random RandGen = new System.Random();
        int RandomNum = 0;

        // Audio objects
        SoundEffect paddleBlipSound;
        SoundEffect bounceBlipSound;
        SoundEffect winnerSound;
        SoundEffect pointSound;
        SoundEffect TurboSound;
        Song introductionSound;

        int P1Score = 0;
        int P2Score = 0;
        SpriteFont ScoreFont;
        bool Contact = false;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            gameMenu = new Menu(Color.Orange, Color.Red, Content.Load<SpriteFont>("scorefont"));
            gameMenu.addMenuItem("", new Vector2(50, 100));
            gameMenu.addMenuItem("New Game", new Vector2(50, 200));
            gameMenu.addMenuItem("Instructions", new Vector2(50, 300));
            gameMenu.addMenuItem("Credits", new Vector2(50, 400));
            gameMenu.addMenuItem("Quit", new Vector2(50, 500));

            MediaPlayer.IsRepeating = true;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        /// 

        Texture2D Pad1;
        Texture2D Pad2;
        Texture2D Balo;
        Texture2D BG;
        Texture2D BG2;

        //Extra balls...
        Texture2D FastBall_1;
        Texture2D FastBall_2;
        Texture2D FastBall_3;
        Texture2D FastBall_4;
        Texture2D FastBall_5;
        Texture2D FastBall_6;

        Vector2 BGPosition = Vector2.Zero;

        Ball bal;
        Pad1 P1;
        Pad2 P2;

        #region LoadContent
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Pad1 = Content.Load<Texture2D>("p1a");
            Pad2 = Content.Load<Texture2D>("p1a");
            Balo = Content.Load<Texture2D>("ball");
            BG = Content.Load<Texture2D>("bg");
            BG2 = Content.Load<Texture2D>("bg2");

            //Extra balls...
            FastBall_1 = Content.Load<Texture2D>("ballfast0");
            FastBall_2 = Content.Load<Texture2D>("ballfast1");
            FastBall_3 = Content.Load<Texture2D>("ballfast2");
            FastBall_4 = Content.Load<Texture2D>("ballfast3");
            FastBall_5 = Content.Load<Texture2D>("ballfast4");
            FastBall_6 = Content.Load<Texture2D>("ballfast5");

            gameMenu.SetText(BG2);

            ScoreFont = Content.Load<SpriteFont>("scorefont");

            Rectangle windowBounds = Window.ClientBounds;

            BGPosition = new Vector2(-115f, 0f);

            bal = new Ball(Balo);
            bal.Position = new Vector2(windowBounds.Width / 2 - Balo.Width, windowBounds.Height / 2 - Balo.Height);

            P1 = new Pad1(Pad1);
            P1.Position = new Vector2(25, windowBounds.Height / 2 - Pad1.Height * 1.5f);

            P2 = new Pad2(Pad2);
            P2.Position = new Vector2(windowBounds.Width - 25 - Pad2.Width,
                windowBounds.Height / 2 - Pad2.Height * 1.5f);

            paddleBlipSound = Content.Load<SoundEffect>("blip2");
            bounceBlipSound = Content.Load<SoundEffect>("blip");
            winnerSound = Content.Load<SoundEffect>("winner");
            pointSound = Content.Load<SoundEffect>("point");
            introductionSound = Content.Load<Song>("intro");
            TurboSound = Content.Load<SoundEffect>("turbo");
        }
        #endregion

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        /// 

        #region Update

        KeyboardState oldState;

        protected override void Update(GameTime gameTime)
        {
            KeyboardState newState = Keyboard.GetState();

            Rectangle windowBounds = Window.ClientBounds;

            if (newState.IsKeyDown(Keys.Escape) && !oldState.IsKeyDown(Keys.Escape) &&
                boolDisplaying == true)
            {
                boolDisplaying = false;
                gameMenu.HideDisplays();
            }

            if (newState.IsKeyDown(Keys.Escape) && !oldState.IsKeyDown(Keys.Escape) ||
                boolPause == true)
            {
                boolPause = true;

                if (boolGameStarted == true)
                {
                    gameMenu.changeMenuItem("", "Resume");
                }
                else if (boolIntroSoundPlayed == false)
                {
                    boolIntroSoundPlayed = true;

                    MediaPlayer.Play(introductionSound);
                }

                if (newState.IsKeyDown(Keys.Down) && !oldState.IsKeyDown(Keys.Down))
                {
                    gameMenu.selectNext();
                }
                else if (newState.IsKeyDown(Keys.Up) && !oldState.IsKeyDown(Keys.Up))
                {
                    gameMenu.selectPrev();
                }

                if (newState.IsKeyDown(Keys.Enter) && !oldState.IsKeyDown(Keys.Enter))
                {
                    MediaPlayer.Stop();

                    if (gameMenu.getSelectedName() == "Resume")
                        boolPause = false;
                    else if (gameMenu.getSelectedName() == "Quit")
                        this.Exit();
                    else if (gameMenu.getSelectedName() == "New Game")
                    {
                        boolPause = false;
                        RestartGame();
                    }
                    else if (gameMenu.getSelectedName() == "Instructions")
                    {
                        gameMenu.DisplayInstructions();
                        boolDisplaying = true;
                    }
                    else if (gameMenu.getSelectedName() == "Credits")
                    {
                        gameMenu.DisplayCredits();
                        boolDisplaying = true;
                    }
                }
            }
            else if (boolPause == false)
            {
                boolGameStarted = true;

                bal.Update(gameTime, graphics, windowBounds, bounceBlipSound);
                P1.Update(graphics);
                P2.Update(graphics);

                //Check for score...
                if (bal.Position.X > graphics.GraphicsDevice.Viewport.Width -
                    Balo.Width)
                {
                    P1Score++;
                    Turbo = false;
                    TurboCount = 0;
                    bal.TurboBallOff();
                    bal.NewText(Balo);

                    if (P1Score < 5)
                        pointSound.Play();

                    if (P1Score == 5)
                    {
                        winnerSound.Play();
                        winner = "Player One Is The Victor!";
                        boolPause = true;
                        gameMenu.changeMenuItem("Resume", "");
                        boolGameStarted = false;
                    }

                    bal.Restart(windowBounds);
                }
                else if (bal.Position.X < 0)
                {
                    P2Score++;
                    Turbo = false;
                    TurboCount = 0;
                    bal.TurboBallOff();
                    bal.NewText(Balo);

                    if (P2Score < 5)
                        pointSound.Play();

                    if (P2Score == 5)
                    {
                        winnerSound.Play();
                        winner = "Player Two Is The Victor!";
                        boolPause = true;
                        gameMenu.changeMenuItem("Resume", "");
                        boolGameStarted = false;
                    }

                    bal.Restart(windowBounds);
                }

                //Ball Strikes Paddle...
                RandomNum = 0;
                if (P1.RectBottom.Intersects(bal.Rect) && Contact == false)
                {
                    paddleBlipSound.Play();
                    bal.BounceAwayBottom(0f, 50f);
                    Contact = true;

                    if (Turbo == false)
                        RandomNum = RandGen.Next(1, 7);
                }
                else if (P1.RectTop.Intersects(bal.Rect) && Contact == false)
                {
                    paddleBlipSound.Play();
                    bal.BounceAwayTop(0f, 50f);
                    Contact = true;

                    if (Turbo == false)
                        RandomNum = RandGen.Next(1, 7);
                }
                else if (P1.Rect.Intersects(bal.Rect) && Contact == false)
                {
                    paddleBlipSound.Play();
                    bal.Bounce(50f, 0f);
                    Contact = true;

                    if (Turbo == false)
                        RandomNum = RandGen.Next(1, 7);
                }
                else if (P2.RectBottom.Intersects(bal.Rect) && Contact == false)
                {
                    paddleBlipSound.Play();
                    bal.BounceAwayBottom(0f, 50f);
                    Contact = true;

                    if (Turbo == false)
                        RandomNum = RandGen.Next(1, 7);
                }
                else if (P2.RectTop.Intersects(bal.Rect) && Contact == false)
                {
                    paddleBlipSound.Play();
                    bal.BounceAwayTop(0f, 50f);
                    Contact = true;

                    if (Turbo == false)
                        RandomNum = RandGen.Next(1, 7);
                }
                else if (P2.Rect.Intersects(bal.Rect) && Contact == false)
                {
                    paddleBlipSound.Play();
                    bal.Bounce(50f, 0f);
                    Contact = true;

                    if(Turbo == false)
                        RandomNum = RandGen.Next(1, 7);
                }
                else if (!P2.Rect.Intersects(bal.Rect) && !P1.Rect.Intersects(bal.Rect))
                    Contact = false;

                //Check for Turbo...
                if (RandomNum == 3 && !Turbo)
                {
                    Turbo = true;
                    TurboSound.Play();
                    RandomNum = 0;
                }

                if (Turbo)
                {
                    if (TurboCount == 0)
                    {
                        bal.NewText(FastBall_1);
                        bal.TurboBall(1.0f, 1.0f);
                    }
                    else if (TurboCount == 7)
                    {
                        bal.NewText(FastBall_2);
                    }
                    else if (TurboCount == 14)
                    {
                        bal.NewText(FastBall_3);
                    }
                    else if (TurboCount == 21)
                    {
                        bal.NewText(FastBall_4);
                    }
                    else if (TurboCount == 28)
                    {
                        bal.NewText(FastBall_5);
                    }
                    else if (TurboCount == 35)
                    {
                        bal.NewText(FastBall_6);
                    }
                    else if (TurboCount == 50)
                    {
                        bal.TurboBall(0.8f, 0.8f);
                    }
                    else if(TurboCount == 100)
                    {
                        bal.TurboBall(0.6f, 0.6f);
                    }
                    else if (TurboCount >= 200)
                    {
                        bal.NewText(Balo);
                        TurboCount = -1;
                        Turbo = false;
                        bal.TurboBallOff();
                    }

                    TurboCount++;
                }



                base.Update(gameTime);
            }

            gameMenu.update(gameTime);
            oldState = newState;
        }
        #endregion

        void RestartGame()
        {
            Rectangle windowBounds = Window.ClientBounds;

            bal.Restart(windowBounds);
            P1.Restart(windowBounds);
            P2.Restart(windowBounds);

            P1Score = 0;
            P2Score = 0;
            winner = "";
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        /// 

        #region Draw
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend);

            if (boolPause == false)
            {
                spriteBatch.Draw(BG, BGPosition, Color.White);
                spriteBatch.DrawString(ScoreFont, P1Score.ToString(),
                    new Vector2(50.0f, 10.0f), Color.White);
                spriteBatch.DrawString(ScoreFont, P2Score.ToString(),
                    new Vector2(graphics.GraphicsDevice.Viewport.Width - 70, 10.0f), Color.White);

                P1.Draw(spriteBatch);
                P2.Draw(spriteBatch);
                bal.Draw(spriteBatch);
            }
            else if (boolDisplaying == false)
            {
                gameMenu.Draw(spriteBatch);
                spriteBatch.DrawString(ScoreFont, winner, new Vector2(graphics.GraphicsDevice.Viewport.Width / 2 - 150, 20.0f), Color.LimeGreen);
            }
            else
                gameMenu.Draw(spriteBatch);

            spriteBatch.End();

            base.Draw(gameTime);
        }
        #endregion
    }
}
